#!/usr/bin/python
from iniparse import ConfigParser
from libtiscompliance.check_backup import *
from libtiscompliance.check_nagios import *
from libtiscompliance.check_network import *
from libtiscompliance.check_windows import *
from libtiscompliance.check_xen import *

cp = ConfigParser()
cp.read("/etc/tis/tis-compliance.ini")
XENIP = cp.get('uwsgi','xenip')
XENPASSWD = cp.get('uwsgi','xenpasswd')
NAGIOSPASSWD = cp.get('uwsgi','nagiospasswd')
NAGIOSUSER = cp.get('uwsgi','nagiosuser')
BACKUPURL = cp.get('uwsgi','backupurl')
ZONE = cp.get('uwsgi','zone')
URL = cp.get('uwsgi','url')

def main():

    nagiosState = {}
    nagiosState['ok'] = 0
    nagiosState['warning'] = 1
    nagiosState['critical'] = 2
    nagiosState['unknown'] = 3
    try:
        #Check nagios configuration
        network_list = get_last_hosts()
        nagios_list = mk_list(ZONE,NAGIOSUSER,NAGIOSPASSWD)
        check_nagios(check_ip_nagios(network_list,nagios_list,nagiosState),nagiosState)

        #Check DNS configuration
        check_nagios(check_dns(network_list,nagiosState),nagiosState)

        #Check Windows Update Settings
        windowsList = srvwindows_list(ZONE,NAGIOSUSER,NAGIOSPASSWD)
        check_nagios(check_windows_update_settings(windowsList,nagiosState),nagiosState)
        
        #Check windows Search Indexer
        check_nagios(check_windows_searchindexer(windowsList,nagiosState), nagiosState)

        #Check VDI names
        check_nagios(check_vdi(XENIP,XENPASSWD,nagiosState),nagiosState)

        #Check ISO library on Xen serveur
        check_nagios(check_sr_iso(XENIP,XENPASSWD,nagiosState),nagiosState)

        #Check Xen tools install on VMs
        check_nagios(check_xentools(XENIP,XENPASSWD,nagiosState),nagiosState)

        #Check tisbackup configuration
        backup_list = tisbackup_list(BACKUPURL)
        check_nagios(check_backup(network_list,backup_list,nagiosState),nagiosState)


        #All checks passed
        print "Ok - all the network is compliant"
        sys.exit(nagiosState['ok'])
    except Exception as e:
        print e
        sys.exit(nagiosState['unknown'])


def check_nagios(result_dict,nagiosState):
    if result_dict['warn'] == nagiosState['unknown']:
        print result_dict['message']
        sys.exit(result_dict['warn'])
    elif result_dict['warn'] == nagiosState['warning']:
        if len(result_dict['list']) > 1 :
            print "Warning - " + ", ".join(result_dict['list']) + result_dict['messages']
            sys.exit(result_dict['warn'])
        elif len(result_dict['list']) == 1:
            print "Warning - " + result_dict['list'][0] + result_dict['message']
            sys.exit(result_dict['warn'])



if __name__ == '__main__':
    main()
