import fcntl
import ipaddr
import struct
import sys
from dns import resolver, reversename

import nmap

from model.manager import *


def get_localhost_mac(ifname):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    info = fcntl.ioctl(s.fileno(), 0x8927, struct.pack('256s', ifname[:15]))
    return ''.join(['%02x:' % ord(char) for char in info[18:24]])[:-1]


def check_dns(ip_list, nagiosState):
    check_ip_list = []
    result_dict = {'warn': nagiosState['ok'], 'title': "DNS"}
    bad_list = []

    try:
        for host in ip_list:
            if not host.exclude:
                check_ip_list.append([host.ip, host.name_ptr, host.ip_forward])
        for ip in check_ip_list:
            if ip[1] == "":
                octet = ip[0].split(".")[2] + "." + ip[0].split(".")[3]
                bad_list.append(octet)
            if ip[2] == "":
                octet = ip[0].split(".")[2] + "." + ip[0].split(".")[3]
                if octet not in bad_list:
                    bad_list.append(octet)
            if ip[0] <> ip[2]:
                octet = ip[0].split(".")[2] + "." + ip[0].split(".")[3]
                if octet not in bad_list:
                    bad_list.append(octet)

        result_dict['list'] = bad_list
        result_dict[
            'messages'] = " are not compliant with dns. Please consult http://srvnagios:8080/netinfo for more details."
        result_dict[
            'message'] = " is not compliant with dns. Please consult http://srvnagios:8080/netinfo for more details."
        if len(bad_list) > 0:
            result_dict['warn'] = nagiosState['warning']


    except StandardError, message:
        result_dict['message'] = "%s : error %s" % (os.path.basename(sys.argv[0]), message)
        result_dict['warn'] = nagiosState['unknown']

    return result_dict

    isIn = False
    for range in NO_CHECK_RANGE:
        if range['from'] <= ip <= range['to']:
            isIn = True
    return isIn


def scan_network(NETWORK, NO_CHECK_RANGE):
    nm = nmap.PortScanner()
    nm.scan(hosts=NETWORK, arguments='-n -p22,135,139,445,80,443,3389,8088', sudo=True)
    inf = get_local_site()[0].interfaces[0]
    for host_info in nm.all_hosts():
        if nm[host_info].state() == 'up':
            guest_os = ""
            if nm[host_info].has_key('osmatch') and len(nm[host_info]['osmatch']) > 0:
                guest_os = nm[host_info]['osmatch'][0]['name']
            ports = [x for x in nm[host_info]['tcp'] if nm[host_info]['tcp'][x]['state'] == "open"]
            ip = ipaddr.IPv4Address(host_info)
            if 'mac' in nm[host_info]['addresses'].keys():
                mac = nm[host_info]['addresses']['mac']
            else:
                mac = get_localhost_mac('eth0')
            ptr_exists = True

            ptr_value = ""
            forward_dns_from_ptr = ""

            # check that PTR of ip address exists 
            try:
                ptr_canonical_addr = reversename.from_address(str(ip))
                ptr_value = str(resolver.query(ptr_canonical_addr, "PTR")[0])
            except:
                ptr_exists = False

                # check that forward dns match pTR
            try:
                if ptr_exists:
                    forward_dns_from_ptr = ''
                    answers = resolver.query(ptr_value, 'A')
                    for rdata in answers:
                        forward_dns_from_ptr = str(rdata)

            except:
                pass

            if isInExcludeRange(ip, NO_CHECK_RANGE):
                exclude = True
            else:
                exclude = False
            add_host(str(ip), mac, ptr_value, forward_dns_from_ptr, exclude, ports, guest_os,interface=inf)


def isInExcludeRange(ip, NO_CHECK_RANGE):
    isIn = False
    for range in NO_CHECK_RANGE:
        if range['from'] <= ip <= range['to']:
            isIn = True
    return isIn
