import os
import sys
import ssl
import XenAPI

if hasattr(ssl, '_create_unverified_context'):
    ssl._create_default_https_context = ssl._create_unverified_context


def check_xentools(xenip, passwd, nagiosState):
    result_dict = {'warn': nagiosState['ok'], 'title': "Xen Tools"}

    session = xen_session(xenip, passwd)
    bad_list = []

    vm_list = session.xenapi.VM.get_all()
    for vm in vm_list:
        if not session.xenapi.VM.get_is_a_template(
                vm) and not "clean_shutdown" in session.xenapi.VM.get_allowed_operations(
            vm) and session.xenapi.VM.get_power_state(vm) == 'Running':
            name = session.xenapi.VM.get_name_label(vm)
            if not "Control domain on host" in name:
                bad_list.append(name)

    result_dict['list'] = bad_list
    result_dict['messages'] = " don't have Xen Tools"
    result_dict['message'] = " doesn't have Xen Tools"
    if len(bad_list) > 0:
        result_dict['warn'] = nagiosState['warning']

    return result_dict


def check_sr_iso(xenip, passwd, nagiosState):
    result_dict = {'list': [], 'title': "SR iso", 'warn': nagiosState['ok']}

    session = xen_session(xenip, passwd)
    srs = []
    for sr in session.xenapi.SR.get_all():
        if "iso" == session.xenapi.SR.get_type(sr) and int(session.xenapi.SR.get_physical_utilisation(sr)) > 0:
            vms = []
            for vdi in session.xenapi.SR.get_VDIs(sr):
                for vbd in session.xenapi.VDI.get_VBDs(vdi):
                    vms += [session.xenapi.VM.get_name_label(session.xenapi.VBD.get_VM(vbd)), ]
            if len(session.xenapi.SR.get_PBDs(sr)) > 0:
                attached = session.xenapi.PBD.get_currently_attached(session.xenapi.SR.get_PBDs(sr)[0])
            else:
                attached = False
            srs += [
                {"name": session.xenapi.SR.get_name_label(sr), "attached": attached, "VM": vms},
            ]

    if len(srs) == 0:
        result_dict['message'] = " haven't an ISO library"
        result_dict['warn'] = nagiosState['ok']
    elif len(srs) > 1:
        result_dict['message'] = " have too many ISO libraries"
        result_dict['warn'] = nagiosState['warning']
    elif len(srs) == 1:
        if len(srs[0]["VM"]) > 0:
            result_dict['message'] = " has an attach ISO"
            result_dict['messages'] = " have an attach ISO"
            result_dict['list'] = srs[0]["VM"]
            result_dict['warn'] = nagiosState['warning']
        elif srs[0]["attached"]:
            result_dict['message'] = " is connected"
            result_dict['list'] = [srs[0]["name"]]
            result_dict['warn'] = nagiosState['warning']
    return result_dict


def xen_session(ip, passwd):
    mastersrvxen = ip
    password_xen = passwd

    session = XenAPI.Session('https://' + mastersrvxen)
    try:
        session.login_with_password('root', password_xen)
    except XenAPI.Failure, error:
        if 'HOST_IS_SLAVE' in error.details:
            msg, ip = error.details
            mastersrvxen = ip
            session = XenAPI.Session('https://' + mastersrvxen)
            session.login_with_password('root', password_xen)
        else:
            raise Exception('Error in XenServer : %s' % error.details)
    return session


def check_vdi(xenip, passwd, nagiosState):
    result_dict = {'title': "VDI", 'warn': nagiosState['ok']}

    try:
        bad_list = zero_vdi_list(xen_session(xenip, passwd))
        result_dict['list'] = bad_list
        result_dict['messages'] = " have an attached VDI named '0'"
        result_dict['message'] = " has an attached VDI named '0'"
        if len(bad_list) > 0:
            result_dict['warn'] = nagiosState['warning']


    except StandardError, message:
        result_dict['message'] = "%s : error %s" % (os.path.basename(sys.argv[0]), message)
        result_dict['warn'] = nagiosState['unknown']

    return result_dict


def zero_vdi_list(session):
    bad_list = []

    vbd_list = session.xenapi.VBD.get_all_records()
    for vbd in vbd_list:
        if not session.xenapi.VBD.get_VDI(vbd) in ["OpaqueRef:NULL", ""]:
            vdi = session.xenapi.VBD.get_VDI(vbd)
            if session.xenapi.VDI.get_name_label(vdi) == "0":
                vm = session.xenapi.VBD.get_VM(vbd)
                if not (session.xenapi.VM.get_is_a_snapshot(vm) or session.xenapi.VM.get_is_a_template(vm)):
                    name = session.xenapi.VM.get_name_label(vm)
                    bad_list.append(name)
    return bad_list
