#!/usr/bin/python
# -*- coding: utf8 -*-
from peewee import *
import os
import socket
import datetime

database_file = "model/netscan.db"
database = SqliteDatabase(database_file)


class BaseModel(Model):
    class Meta:
        database = database


class Date(Model):
    date_insert = DateTimeField()

    class Meta:
        database = database  #

def add_date():
    with database.transaction():
        adate = Date.create( date_insert = datetime.datetime.now())
    return adate

class Site(Model):
    hostname = CharField()

    class Meta:
        database = database


class Interface(Model):
    name = CharField()
    site = ForeignKeyField(Site, related_name='interfaces')

    class Meta:
        database = database


class Host(Model):
    ip = CharField(null=True)
    mac = CharField(null=True)
    name_ptr = CharField(null=True)
    ip_forward = CharField(null=True)
    exclude = BooleanField(null=True)
    ports = CharField(null=True)
    guest_os = CharField(null=True)
    interface = ForeignKeyField(Interface, related_name='hosts')

    class Meta:
        database = database


class Relationship(BaseModel):
    host = ForeignKeyField(Host)
    date = ForeignKeyField(Date)

    class Meta:
        indexes = (
            (('date', 'host'), True),
        )


def get_last_hosts():
    last_date = Date.select().order_by(Date.id.desc())
    if last_date.count() > 0:
        hosts = Host.select().join(Relationship).join(Date).where(Date.date_insert == last_date.get().date_insert)
        return sorted(hosts, key=lambda item: socket.inet_aton(item.ip))
    else:
        return []


def add_host(ip, mac, name_ptr=None, ip_forward=None, exclude=None, ports=None, guest_os=None, interface=None):
    existing_host = (Host.select().where(
        (Host.ip == ip) & (Host.mac == mac) & (Host.name_ptr == name_ptr) & (Host.ip_forward == ip_forward) & (
        Host.ports == str(ports)) & (Host.exclude == exclude)))
    try:
        with database.transaction():
            if existing_host.count() > 0:
                host = existing_host.get()
            else:
                host = Host.create(
                    ip=ip,
                    mac=mac,
                    name_ptr=name_ptr,
                    ip_forward=ip_forward,
                    exclude=exclude,
                    ports=ports,
                    guest_os=guest_os,
                    interface=interface
                )
            if interface.site.hostname == "local":
                rel = Relationship.create(host=host, date=Date.select().order_by(Date.id.desc()).get())

    except Exception as e:
        print u"Error: %s" % (e)


def create_tables():
    if not os.path.exists(database_file):
        database.connect()
        database.create_tables([Site, Interface, Host, Relationship, Date, ])
        site = add_site("local")
        add_interface("eth0", site)


def archive_hosts():
    add_date()

def get_local_site():
    return Site.select().where(Site.hostname == "local")

def get_sites():
    return Site.select().where(Site.hostname != "local")


def add_site(name):
    try:
        with database.transaction():
            site = Site.create(
                hostname=name,
            )
        return site
    except Exception as e:
        print u"Error: %s" % (e)


def del_site(site):
    with database.transaction():
        site.delete_instance(recursive=True)


def add_interface(name, site):
    try:
        with database.transaction():
            interface = Interface.create(
                name=name,
                site=site
            )

        return interface
    except Exception as e:
        print u"Error: %s" % (e)
