import re
import simplejson as json
from iniparse import ConfigParser

from flask import Flask, render_template, redirect, request

from libtiscompliance.check_backup import *
from libtiscompliance.check_nagios import *
from libtiscompliance.check_network import *
from libtiscompliance.check_windows import *
from libtiscompliance.check_wapt import *
from libtiscompliance.check_sites import *
from libtiscompliance.check_xen import *
from model.manager import *

cp = ConfigParser()
cp.read("/etc/tis/tis-compliance.ini")

NETWORK = cp.get('uwsgi','network') if cp.has_option('uwsgi','network') else ""
NO_CHECK_RANGE = []
for no_check in re.findall("no_check_[^']*", str(cp.options('uwsgi'))):
    for range in cp.get('uwsgi', no_check).split(','):
        if len(range.split('-')) > 1:
            NO_CHECK_RANGE.append({ 'to': ipaddr.IPv4Address(range.split('-')[1]) , 'from': ipaddr.IPv4Address(range.split('-')[0]) })
        else:
            NO_CHECK_RANGE.append({ 'to': ipaddr.IPv4Address(range.split('-')[0]) , 'from': ipaddr.IPv4Address(range.split('-')[0]) })

XENIP = cp.get('uwsgi','xenip') if cp.has_option('uwsgi','xenip') else ""
NAGIOSPASSWD = cp.get('uwsgi','nagiospasswd') if cp.has_option('uwsgi','nagiospasswd') else ""
NAGIOSUSER = cp.get('uwsgi','nagiosuser') if cp.has_option('uwsgi','nagiosuser') else ""
BACKUPURL = cp.get('uwsgi','backupurl') if cp.has_option('uwsgi','backupurl') else ""
ZONE = cp.get('uwsgi','zone') if cp.has_option('uwsgi','zone') else ""
SITES = cp.get('uwsgi','sites').split(',') if cp.has_option('uwsgi','sites') else ""


app = Flask(__name__)
app.config['PROPAGATE_EXCEPTIONS'] = True

create_tables()

@app.route('/netscan')
def netscan():
    archive_hosts()
    scan_network(NETWORK, NO_CHECK_RANGE)
    return redirect('/')

@app.route('/')
def net_stat():
    return render_template('list.html', ip_list = get_last_hosts())

@app.route('/sites')
def sites():
    return render_template('sites.html', sites = get_sites())

@app.route('/sitesscan')
def sitesscan():
    archive_sites(SITES)
    return redirect('/sites')

@app.route('/vdi', methods=['GET', 'POST'])
def vdi():
    session = xen_session(XENIP,XENPASSWD)
    bad_vdi_list = zero_vdi_list(session)
    if request.method == 'GET':
        return render_template('vdi.html', vdi_list = bad_vdi_list)
    else:
        for row in bad_vdi_list:
            vm = session.xenapi.VM.get_by_name_label(row)
            vbds = session.xenapi.VM.get_VBDs(vm[0])
            for vbd in vbds:
                vdi = session.xenapi.VBD.get_VDI(vbd)
                if not "NULL" in vdi:
                    session.xenapi.VDI.set_name_label(vdi, row)
        return render_template("vdi.html", message="All bad VDI have been renamed successfully")



@app.route('/nagios')
def check_warning():
    nagiosState = {}
    nagiosState['ok'] = 0
    nagiosState['warning'] = 1
    nagiosState['critical'] = 2
    nagiosState['unknown'] = 3
    
    try:
        #Check nagios configuration
        network_list = get_last_hosts()
        nagios_list = mk_list(ZONE,NAGIOSUSER,NAGIOSPASSWD)
        nagios_warning = check_ip_nagios(network_list,nagios_list,nagiosState)

        #Check DNS configuration
        dns_warning = check_dns(network_list,nagiosState)

        #Check VDI names
        vdi_warning = check_vdi(XENIP,XENPASSWD,nagiosState)

        # Check WAPT
        wapt_warning = check_wapt(network_list)
    
        #Check windows update settings
        windowsList = srvwindows_list(ZONE,NAGIOSUSER,NAGIOSPASSWD)
        windows_update_settings_warning = check_windows_update_settings(windowsList,nagiosState)
        
        #Check windows Search Indexer
        windows_searchindexer_warning  = check_windows_searchindexer(windowsList,nagiosState)

        #Check ISO library on Xen serveur
        sr_iso_warning = check_sr_iso(XENIP,XENPASSWD,nagiosState)

        #Check Xen tools install on VMs
        xentools_warning = check_xentools(XENIP,XENPASSWD,nagiosState)


        #Check tisbackup configuration
        backup_list = tisbackup_list(BACKUPURL)
        backup_warning = check_backup(network_list,backup_list,nagiosState)
        
        errors_list = (nagios_warning, dns_warning, vdi_warning, wapt_warning, windows_update_settings_warning, windows_searchindexer_warning, sr_iso_warning, xentools_warning, backup_warning)
        
        return render_template("nagios.html", errors = errors_list )
    except Exception as e:
        return str(e)


@app.route("/check_nagios")
def check_nagios():
    ip_list = get_last_hosts()
    return json.dumps(ip_list)

if __name__ == "__main__":
    app.debug = True
    #scan_network(NETWORK, NO_CHECK_RANGE)
    app.config['PROPAGATE_EXCEPTIONS'] = True
    app.run(host='0.0.0.0',port=8080, debug=True)
